import unittest

from prometheus_client import samples


class TestSamples(unittest.TestCase):
    def test_gt(self):
        self.assertEqual(samples.Timestamp(1, 1) > samples.Timestamp(1, 1), False)
        self.assertEqual(samples.Timestamp(1, 1) > samples.Timestamp(1, 2), False)
        self.assertEqual(samples.Timestamp(1, 1) > samples.Timestamp(2, 1), False)
        self.assertEqual(samples.Timestamp(1, 1) > samples.Timestamp(2, 2), False)
        self.assertEqual(samples.Timestamp(1, 2) > samples.Timestamp(1, 1), True)
        self.assertEqual(samples.Timestamp(2, 1) > samples.Timestamp(1, 1), True)
        self.assertEqual(samples.Timestamp(2, 2) > samples.Timestamp(1, 1), True)

    def test_lt(self):
        self.assertEqual(samples.Timestamp(1, 1) < samples.Timestamp(1, 1), False)
        self.assertEqual(samples.Timestamp(1, 1) < samples.Timestamp(1, 2), True)
        self.assertEqual(samples.Timestamp(1, 1) < samples.Timestamp(2, 1), True)
        self.assertEqual(samples.Timestamp(1, 1) < samples.Timestamp(2, 2), True)
        self.assertEqual(samples.Timestamp(1, 2) < samples.Timestamp(1, 1), False)
        self.assertEqual(samples.Timestamp(2, 1) < samples.Timestamp(1, 1), False)
        self.assertEqual(samples.Timestamp(2, 2) < samples.Timestamp(1, 1), False)


if __name__ == '__main__':
    unittest.main()



import gzip
from unittest import TestCase
from wsgiref.util import setup_testing_defaults

from prometheus_client import CollectorRegistry, Counter, make_wsgi_app
from prometheus_client.exposition import _bake_output, CONTENT_TYPE_LATEST


class WSGITest(TestCase):
    def setUp(self):
        self.registry = CollectorRegistry()
        self.captured_status = None
        self.captured_headers = None
        # Setup WSGI environment
        self.environ = {}
        setup_testing_defaults(self.environ)

    def capture(self, status, header):
        self.captured_status = status
        self.captured_headers = header

    def increment_metrics(self, metric_name, help_text, increments):
        c = Counter(metric_name, help_text, registry=self.registry)
        for _ in range(increments):
            c.inc()

    def assert_outputs(self, outputs, metric_name, help_text, increments, compressed):
        self.assertEqual(len(outputs), 1)
        if compressed:
            output = gzip.decompress(outputs[0]).decode(encoding="utf-8")
        else:
            output = outputs[0].decode('utf8')
        # Status code
        self.assertEqual(self.captured_status, "200 OK")
        # Headers
        num_of_headers = 2 if compressed else 1
        self.assertEqual(len(self.captured_headers), num_of_headers)
        self.assertIn(("Content-Type", CONTENT_TYPE_LATEST), self.captured_headers)
        if compressed:
            self.assertIn(("Content-Encoding", "gzip"), self.captured_headers)
        # Body
        self.assertIn("# HELP " + metric_name + "_total " + help_text + "\n", output)
        self.assertIn("# TYPE " + metric_name + "_total counter\n", output)
        self.assertIn(metric_name + "_total " + str(increments) + ".0\n", output)

    def validate_metrics(self, metric_name, help_text, increments):
        """
        WSGI app serves the metrics from the provided registry.
        """
        self.increment_metrics(metric_name, help_text, increments)
        # Create and run WSGI app
        app = make_wsgi_app(self.registry)
        outputs = app(self.environ, self.capture)
        # Assert outputs
        self.assert_outputs(outputs, metric_name, help_text, increments, compressed=False)

    def test_report_metrics_1(self):
        self.validate_metrics("counter", "A counter", 2)

    def test_report_metrics_2(self):
        self.validate_metrics("counter", "Another counter", 3)

    def test_report_metrics_3(self):
        self.validate_metrics("requests", "Number of requests", 5)

    def test_report_metrics_4(self):
        self.validate_metrics("failed_requests", "Number of failed requests", 7)

    def test_favicon_path(self):
        from unittest.mock import patch

        # Create mock to enable counting access of _bake_output
        with patch("prometheus_client.exposition._bake_output", side_effect=_bake_output) as mock:
            # Create and run WSGI app
            app = make_wsgi_app(self.registry)
            # Try accessing the favicon path
            favicon_environ = dict(self.environ)
            favicon_environ['PATH_INFO'] = '/favicon.ico'
            outputs = app(favicon_environ, self.capture)
            # Test empty response
            self.assertEqual(outputs, [b''])
            self.assertEqual(mock.call_count, 0)
            # Try accessing normal paths
            app(self.environ, self.capture)
            self.assertEqual(mock.call_count, 1)

    def test_gzip(self):
        # Increment a metric
        metric_name = "counter"
        help_text = "A counter"
        increments = 2
        self.increment_metrics(metric_name, help_text, increments)
        app = make_wsgi_app(self.registry)
        # Try accessing metrics using the gzip Accept-Content header.
        gzip_environ = dict(self.environ)
        gzip_environ['HTTP_ACCEPT_ENCODING'] = 'gzip'
        outputs = app(gzip_environ, self.capture)
        # Assert outputs are compressed.
        self.assert_outputs(outputs, metric_name, help_text, increments, compressed=True)

    def test_gzip_disabled(self):
        # Increment a metric
        metric_name = "counter"
        help_text = "A counter"
        increments = 2
        self.increment_metrics(metric_name, help_text, increments)
        # Disable compression explicitly.
        app = make_wsgi_app(self.registry, disable_compression=True)
        # Try accessing metrics using the gzip Accept-Content header.
        gzip_environ = dict(self.environ)
        gzip_environ['HTTP_ACCEPT_ENCODING'] = 'gzip'
        outputs = app(gzip_environ, self.capture)
        # Assert outputs are not compressed.
        self.assert_outputs(outputs, metric_name, help_text, increments, compressed=False)



#!/usr/bin/env python


from ..utils import floatToGoString

CONTENT_TYPE_LATEST = 'application/openmetrics-text; version=1.0.0; charset=utf-8'
"""Content type of the latest OpenMetrics text format"""


def _is_valid_exemplar_metric(metric, sample):
    if metric.type == 'counter' and sample.name.endswith('_total'):
        return True
    if metric.type in ('histogram', 'gaugehistogram') and sample.name.endswith('_bucket'):
        return True
    return False


def generate_latest(registry):
    '''Returns the metrics from the registry in latest text format as a string.'''
    output = []
    for metric in registry.collect():
        try:
            mname = metric.name
            output.append('# HELP {} {}\n'.format(
                mname, metric.documentation.replace('\\', r'\\').replace('\n', r'\n').replace('"', r'\"')))
            output.append(f'# TYPE {mname} {metric.type}\n')
            if metric.unit:
                output.append(f'# UNIT {mname} {metric.unit}\n')
            for s in metric.samples:
                if s.labels:
                    labelstr = '{{{0}}}'.format(','.join(
                        ['{}="{}"'.format(
                            k, v.replace('\\', r'\\').replace('\n', r'\n').replace('"', r'\"'))
                            for k, v in sorted(s.labels.items())]))
                else:
                    labelstr = ''
                if s.exemplar:
                    if not _is_valid_exemplar_metric(metric, s):
                        raise ValueError(f"Metric {metric.name} has exemplars, but is not a histogram bucket or counter")
                    labels = '{{{0}}}'.format(','.join(
                        ['{}="{}"'.format(
                            k, v.replace('\\', r'\\').replace('\n', r'\n').replace('"', r'\"'))
                            for k, v in sorted(s.exemplar.labels.items())]))
                    if s.exemplar.timestamp is not None:
                        exemplarstr = ' # {} {} {}'.format(
                            labels,
                            floatToGoString(s.exemplar.value),
                            s.exemplar.timestamp,
                        )
                    else:
                        exemplarstr = ' # {} {}'.format(
                            labels,
                            floatToGoString(s.exemplar.value),
                        )
                else:
                    exemplarstr = ''
                timestamp = ''
                if s.timestamp is not None:
                    timestamp = f' {s.timestamp}'
                output.append('{}{} {}{}{}\n'.format(
                    s.name,
                    labelstr,
                    floatToGoString(s.value),
                    timestamp,
                    exemplarstr,
                ))
        except Exception as exception:
            exception.args = (exception.args or ('',)) + (metric,)
            raise

    output.append('# EOF\n')
    return ''.join(output).encode('utf-8')
